-- Greg Stitt
-- University of Florida
-- EEL 5934/4930 Reconfigurable Computing
--
-- File: ctrl.vhd
--
-- Description: This file implements the controller for the Fibonacci
-- calculator
--
-- Comments: The FSM was kept simple for ease of understanding. It can
-- certainly be optimized if you want to put in the extra effort.

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity CTRL is
  port( clk, rst, go : in  std_logic;
        done         : out std_logic;

        -- control signals used/generated by the datapath
        i_sel, x_sel, y_sel               : out std_logic;
        i_ld, x_ld, y_ld, n_ld, result_ld : out std_logic;
        i_le_n                            : in  std_logic );
end CTRL;

architecture bhv of CTRL is

  type STATE_TYPE is (START, WAIT_0, WAIT_1, INIT, LOOP_COND,
                      LOOP_BODY, RESULT);
  signal state, next_state : STATE_TYPE;

begin

  -- state register
  process (clk, rst)
  begin
    if (rst = '1') then
      state <= START;
    elsif (clk = '1' and clk'event) then
      state <= next_state;
    end if;
  end process;

  -- next state logic
  process( go, i_le_n, state )
  begin

    case state is
      when START =>

        done      <= '0';
        i_sel     <= '-';
        x_sel     <= '-';
        y_sel     <= '-';
        i_ld      <= '-';
        x_ld      <= '-';
        y_ld      <= '-';
        n_ld      <= '-';
        result_ld <= '0';

        if (go = '0') then
          next_state <= WAIT_1;
        else
          next_state <= START;
        end if;

      when WAIT_1 =>

        done      <= '0';
        i_sel     <= '-';
        x_sel     <= '-';
        y_sel     <= '-';
        i_ld      <= '-';
        x_ld      <= '-';
        y_ld      <= '-';
        n_ld      <= '-';
        result_ld <= '0';

        if (go = '1') then
          next_state <= INIT;
        else
          next_state <= WAIT_1;
        end if;

      when INIT =>

        done      <= '0';
        i_sel     <= '1';
        x_sel     <= '1';
        y_sel     <= '1';
        i_ld      <= '1';
        x_ld      <= '1';
        y_ld      <= '1';
        n_ld      <= '1';
        result_ld <= '0';

        next_state <= LOOP_COND;

      when LOOP_COND =>

        done      <= '0';
        i_sel     <= '-';
        x_sel     <= '-';
        y_sel     <= '-';
        i_ld      <= '0';
        x_ld      <= '0';
        y_ld      <= '0';
        n_ld      <= '0';
        result_ld <= '0';

        if (i_le_n = '1') then
          next_state <= LOOP_BODY;
        else
          next_state <= RESULT;
        end if;

      when LOOP_BODY =>

        done      <= '0';
        i_sel     <= '0';
        x_sel     <= '0';
        y_sel     <= '0';
        i_ld      <= '1';
        x_ld      <= '1';
        y_ld      <= '1';
        n_ld      <= '0';
        result_ld <= '0';

        next_state <= LOOP_COND;

      when RESULT =>
        done      <= '0';
        i_sel     <= '-';
        x_sel     <= '-';
        y_sel     <= '-';
        i_ld      <= '-';
        x_ld      <= '-';
        y_ld      <= '-';
        n_ld      <= '-';
        result_ld <= '1';

        next_state <= WAIT_0;

      when WAIT_0 =>
        done      <= '1';
        i_sel     <= '-';
        x_sel     <= '-';
        y_sel     <= '-';
        i_ld      <= '-';
        x_ld      <= '-';
        y_ld      <= '-';
        n_ld      <= '-';
        result_ld <= '0';

        if (go = '0') then
          next_state <= WAIT_1;
        else
          next_state <= WAIT_0;
        end if;

      when others => null;
    end case;

  end process;
end bhv;
