-- Greg Stitt
-- University of Florida
-- EEL 5934/4930 Reconfigurable Computing
--
-- File: fib.vhd
--
-- Description: This file implements the top level entity for the Fibonacci
-- calculator. The entity is a structural description connecting the controller
-- and datapath (FSM+D)

library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity FIB is
  generic (width :     natural := 32);
  port( clk      : in  std_logic;
        rst      : in  std_logic;
        go       : in  std_logic;
        n        : in  unsigned(width-1 downto 0);
        result   : out unsigned(width-1 downto 0);
        done     : out std_logic );
end FIB;

architecture str of FIB is

  component CTRL
    port( clk, rst, go : in  std_logic;
          done         : out std_logic;

          -- control signals used/generated by the datapath
          i_sel, x_sel, y_sel               : out std_logic;
          i_ld, x_ld, y_ld, n_ld, result_ld : out std_logic;
          i_le_n                            : in  std_logic );
  end component;

  component DATAPATH
    generic (width :     natural := 32);
    port(clk, rst  : in  std_logic;
         n         : in  unsigned(width-1 downto 0);
         result    : out unsigned(width-1 downto 0);

         -- control signals

         i_sel, x_sel, y_sel               : in  std_logic;
         i_ld, x_ld, y_ld, n_ld, result_ld : in  std_logic;
         i_le_n                            : out std_logic );
  end component;

  -- signals connecting the controller and datapath
  signal i_sel, x_sel, y_sel               : std_logic;
  signal i_ld, x_ld, y_ld, n_ld, result_ld : std_logic;
  signal i_le_n                            : std_logic;

begin

  CTR1 : CTRL port map (clk, rst, go, done,
                        i_sel, x_sel, y_sel,
                        i_ld, x_ld, y_ld, n_ld, result_ld,
                        i_le_n);

  DP1 : DATAPATH generic map (width)
    port map (clk, rst, n, result,
              i_sel, x_sel, y_sel,
              i_ld, x_ld, y_ld, n_ld, result_ld,
              i_le_n);

end str;
