-- Greg Stitt
-- University of Florida
-- EEL 5934/4930 Reconfigurable Computing
--
-- File: xilinx_wrapper.vhd
--
-- Description: This file is a wrapper for the top level entity. The ISE
-- simulator seems to only like std_logic at the top level, so this entity
-- simply converts the std_logic_vector inputs/outputs to unsigned
-- inputs/outputs for the fib entity.
--
-- Also, the ISE simulator does not like generics in the top level entity, so
-- the wrapper also specifies the width of operations in the FIB entity.


library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

entity XILINX_WRAPPER is
  port( clk    : in  std_logic;
        rst    : in  std_logic;
        go     : in  std_logic;
        n      : in  std_logic_vector(31 downto 0);
        result : out std_logic_vector(31 downto 0);
        done   : out std_logic );
end XILINX_WRAPPER;

architecture str of XILINX_WRAPPER is

  component FIB
    generic (width     :     natural := 32);
    port( clk, rst, go : in  std_logic;
          n            : in  unsigned(width-1 downto 0);
          result       : out unsigned(width-1 downto 0);
          done         : out std_logic );
  end component;

  signal temp_result : unsigned(31 downto 0);

begin

  W1 : FIB generic map (32) port map (clk, rst, go, unsigned(n), temp_result, done);
  result <= std_logic_vector( temp_result );

end str;
